%THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANY OF ANY KIND, AND ITS AUTHOR
%AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH (JAIR) AND JAIR'S
%PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL WARRANTIES, INCLUDING BUT NOT 
%LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%PURPOSE, AND ANY WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY
%AND RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR NOR JAIR, 
%NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR DAMAGES OF ANY KIND
%RESULTING FROM ITS USE. WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, NEITHER 
%THE AUTHOR, NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WARRANT THAT THE
%SOURCE CODE WILL BE ERROR-FREE, WILL OPERATE WITHOUT INTERRUPTION, OR WILL MEET 
%THE NEEDS OF THE USER.

function [critical_points, data] = buildkernelranges(X,h,lowerbound,upperbound)
% SUMMARY: Kernel regression functions are essentially piecewise.  This
%   function finds the critical points that separate portions of the
%   piecewise function.  It also defines a structure describing the data
%   that define each piecewise portion.
% INPUT: X is a sorted vector of data kernel regression is to be performed 
%   with. h is the kernel bandwidth.  lowerbound and upperbound define the
%   interval X may take values in.
% OUTPUT: critical_points is a vector of numbers at which the kernel
%   regression changes from one piecewise portion to another. data is a
%   cell containing the observations that define each piecewise portion

N = length(X);
critical_points = zeros(1,2*length(X));
critical_points(1) = lowerbound;
critical_points(2) = upperbound;
i = 3;
for k=1:N
    currentlower = X(k)-h;
    currentupper = X(k)+h;
    if currentlower > lowerbound
        critical_points(i) = currentlower;
        i = i + 1;
    end
    if currentupper < upperbound
        critical_points(i) = currentupper;
        i = i + 1;
    end
end
critical_points = critical_points(1:i-1);
% rounding to 3 places avoids some computation issues later
critical_points = round(critical_points*1000)/1000;
critical_points = sort(critical_points);
critical_points = unique(critical_points);
M = length(critical_points);
data = cell(M-1,1);
for k=1:M-1;
    data{k} = (X-h < critical_points(k+1) & (X+h) > critical_points(k));
end
