%THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANY OF ANY KIND, AND ITS AUTHOR
%AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH (JAIR) AND JAIR'S
%PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL WARRANTIES, INCLUDING BUT NOT 
%LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%PURPOSE, AND ANY WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY
%AND RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR NOR JAIR, 
%NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR DAMAGES OF ANY KIND
%RESULTING FROM ITS USE. WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, NEITHER 
%THE AUTHOR, NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WARRANT THAT THE
%SOURCE CODE WILL BE ERROR-FREE, WILL OPERATE WITHOUT INTERRUPTION, OR WILL MEET 
%THE NEEDS OF THE USER.

function [a,b,c] = determinepartialproductcoefficients(X,Y,h,state)
% SUMMARY: The numerator of the derivative of an epanechnikov product
%   kernel is a piecewise function with each portion being quadratic.  This
%   function finds the coefficients of that quadratic for the purpose of
%   finding a maximum.
% INPUT: X is a matrix with each row describing a state-action pair.  Y is
%    vector of values for each state-action pair. h is the kernel
%    bandwidth.  state is a vector with state variable values for the state
%    an optimal action is being searched for
% OUTPUT: a,b, and c are the coefficients ax^2 + bx + c for the quadratic

[n, dim] = size(X);
hthree = h^(-3);
c_1 = -.75*hthree;
c_2 = -1.5*hthree*X;
c_3 = .75*h^(-1) - .75*hthree*X;
c_4 = zeros(n,1);
for i=1:n
    product = 1;
    for j=1:dim-1
        product = product*(c_1*state(j)^2 + c_2(i,j)*state(j) + c_3(i,j));
    end
    c_4(i) = product;
end
sumc2c4y = sum(c_2(:,dim).*c_4.*Y);
sumc4    = sum(c_4);
sumc4y   = sum(c_4.*Y);
sumc2c4  = sum(c_2(:,dim).*c_4);
sumc3c4  = sum(c_3(:,dim).*c_4);
sumc3c4y = sum(c_3(:,dim).*c_4.*Y);
a = -c_1*sumc2c4y + c_1*sumc4;
b = 2*c_1*sumc3c4*sumc4y - 2*c_1*sumc3c4y*sumc4;
c = sumc3c4*sumc2c4y - sumc3c4y*sumc2c4;
