%THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANY OF ANY KIND, AND ITS AUTHOR
%AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH (JAIR) AND JAIR'S
%PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL WARRANTIES, INCLUDING BUT NOT 
%LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%PURPOSE, AND ANY WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY
%AND RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR NOR JAIR, 
%NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR DAMAGES OF ANY KIND
%RESULTING FROM ITS USE. WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, NEITHER 
%THE AUTHOR, NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WARRANT THAT THE
%SOURCE CODE WILL BE ERROR-FREE, WILL OPERATE WITHOUT INTERRUPTION, OR WILL MEET 
%THE NEEDS OF THE USER.


function Y_hat = partialproductkernelregression(X, Y, X_fixed, bandwidth, X_var)
% SUMMARY:  This function allows the calculation of kernel regression
%   estimates when some variables (state variables) are kept fixed and the
%   remaining variables (action variables) are allowed to vary
% INPUT: X is a matrix of input variable values. Y is a vector of output
%   variable values.  X and Y are the data the kernel regression is
%   performed with.  X_fixed is a vector describing a single state.
%   bandwidth is the kernel bandwidth to be used.  X_var is a matrix with
%   each row containing action variable values
% OUTPUT: Y_hat is a vector with length equal to the number of columns of
%   X_var.  Each value is a kernel regression estimate using X_fixed as the
%   state and the different values of X_var as the actions.

[n,dim] = size(X);
statedim = length(X_fixed);
[m, ~] = size(X_var);
Y_hat = zeros(1,m);
for j=1:m    
    num = 0;
    den = 0;
    for i=1:n
        productkernel = 1;
        for k=1:statedim
            productkernel = productkernel*epanechnikov(bandwidth,X_fixed(k)-X(i,k));
        end
        for k=(statedim+1):dim            
            productkernel = productkernel*epanechnikov(bandwidth,X_var(j,k-statedim)-X(i,k));
        num = num + productkernel*Y(i);
        den = den + productkernel;
        end
    end
    Y_hat(j) = num/den;
end
if (isnan(Y_hat))
    Y_hat = 0;
end
