%THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANY OF ANY KIND, AND ITS AUTHOR
%AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH (JAIR) AND JAIR'S
%PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL WARRANTIES, INCLUDING BUT NOT 
%LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%PURPOSE, AND ANY WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY
%AND RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR NOR JAIR, 
%NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR DAMAGES OF ANY KIND
%RESULTING FROM ITS USE. WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, NEITHER 
%THE AUTHOR, NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WARRANT THAT THE
%SOURCE CODE WILL BE ERROR-FREE, WILL OPERATE WITHOUT INTERRUPTION, OR WILL MEET 
%THE NEEDS OF THE USER.

function [X, Y] = run_experiment()
% SUMMARY: This is the master function for learning solutions to the
%   Mountain Car problem
% INPUT: none required, as parameters are pulled from the config object
% OUTPUT: X is N x 3 matrix of state-actions, where N is the number of 
%   observations.  The first column consists of positions, the second of 
%   velocities, and the third for actions/forces. Y is an N x 1 vector of 
%   state-action values

% config is a configuration object containing all problem parameters
config = classconfig();
X = zeros(config.iterations,3);
rewards = zeros(config.iterations,1);
Y = zeros(config.iterations,1);

% Begin by creating a set of random but successful episodes to seed with
% i is a counter for successful seeding episodes
i = 0;
% Initialize state 
state = [-.5, 0];
% k is a counter that keeps track of how many observations have been
% recorded
k=1;
while i < config.seed_episodes
    % trajectory contains [state, action, reward, next_state]
    trajectory = zeros(500,6);
    exploring = true;
    j = 1;
    while exploring
        % In this section, all actions are randomly chosen
        r = rand();
        action = config.action_bounds(1) +...
            (config.action_bounds(2) - config.action_bounds(1))*r;
        % See how the system transitions
        [next_state, reward, fail_flag, success_flag] = transition_reward(state,action);
        % Record information about transition
        trajectory(j,:) = [state, action, reward, next_state];        
        if success_flag
            % Truncate trajectory, stop exploring, count the success
            trajectory = trajectory(1:j,:);
            exploring = false;
            state = [-.5, 0];
            i = i + 1;
        elseif fail_flag || j > 500
            % If episode failed or took too long, reset trajectory
            trajectory = zeros(500,6);            
            state = [-.5, 0];
            j = 1;
        else
            state = next_state;
            j = j+1;
        end
    end
    [trajectory_length, ~] = size(trajectory);
    % replay experiences in reverse order to maximize learning
    X(k:k+trajectory_length-1,:) = trajectory(trajectory_length:-1:1,1:3);    
    rewards(k:k+trajectory_length-1) = trajectory(trajectory_length:-1:1,4);    
    for l=1:trajectory_length        
        % Use all previous observations to calculate value of next state
        [value,~] = findpartialproductkernelmax(...
            X(1:k+l-2,:),Y(1:k+l-2),config.bandwidth,trajectory(l,5:6),...
            config.action_bounds(1),config.action_bounds(2));        
        Y(k+l-1) = rewards(k+l-1) + config.gamma*value;        
    end
    k = k + trajectory_length;    
end
% Seeding material is done, now with epsilon-greedy exploration.
% Run until enough observations recorded
while k <= config.iterations
    r = rand();
    if r < config.exploration_parameter
        r = rand();
        action = -4 + 8*r;
        [next_state, reward, fail_flag, success_flag] =...
            transition_reward(state,action);        
        [value, ~] = findpartialproductkernelmax(...
            X(1:k-1,:),Y(1:k-1),config.bandwidth,next_state,...
            config.action_bounds(1),config.action_bounds(2));        
        sample_bellman = reward + config.gamma*value;
        X(k,:) = [state action];
        Y(k) = sample_bellman;        
        k = k + 1;        
        if fail_flag || success_flag            
            state = [-.5,0];
        else
            state = next_state;
        end
        % reduce exploration parameter value so that exploration vanishes
        % over time
        config.exploration_parameter = config.exploration_parameter*.999;
    else
        % Do not explore, choose best known action
        [~, action] = findpartialproductkernelmax(...
            X(1:k-1,:),Y(1:k-1),config.bandwidth,state,-4,4);            
        [next_state, reward, fail_flag, success_flag] =...
            transition_reward(state,action);        
        [value, ~] = findpartialproductkernelmax(...
            X(1:k-1,:),Y(1:k-1),config.bandwidth,next_state,-4,4);        
        sample_bellman = reward + config.gamma*value;
        X(k,:) = [state action];
        Y(k) = sample_bellman;        
        k = k + 1
        if fail_flag || success_flag
            state = [-.5,0];
        else
            state = next_state;
        end
    end
end
F1 = policy_test(X,Y)
