%THIS SOURCE CODE IS SUPPLIED "AS IS" WITHOUT WARRANY OF ANY KIND, AND ITS AUTHOR
%AND THE JOURNAL OF ARTIFICIAL INTELLIGENCE RESEARCH (JAIR) AND JAIR'S
%PUBLISHERS AND DISTRIBUTORS, DISCLAIM ANY AND ALL WARRANTIES, INCLUDING BUT NOT 
%LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
%PURPOSE, AND ANY WARRANTIES OR NON INFRINGEMENT.  THE USER ASSUMES ALL LIABILITY
%AND RESPONSIBILITY FOR USE OF THIS SOURCE CODE, AND NEITHER THE AUTHOR NOR JAIR, 
%NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WILL BE LIABLE FOR DAMAGES OF ANY KIND
%RESULTING FROM ITS USE. WITHOUT LIMITING THE GENERALITY OF THE FOREGOING, NEITHER 
%THE AUTHOR, NOR JAIR, NOR JAIR'S PUBLISHERS AND DISTRIBUTORS, WARRANT THAT THE
%SOURCE CODE WILL BE ERROR-FREE, WILL OPERATE WITHOUT INTERRUPTION, OR WILL MEET 
%THE NEEDS OF THE USER.

function [next_state, reward, fail_flag, success_flag] = transition_reward(state, action)
% SUMMARY: This function defines the Mountain Car system dynamics
% INPUT: state = [position, velocity], action = [force]
% OUTPUT: next_state = [next_position, next_velocity], reward is
%   self explanatory, fail_flag and success_flag are booleans
%   indicating whether a goal or penalty state has been reached

%config is an object containing problem parameters
config = classconfig();
fail_flag = false;
success_flag = false;
num_of_intermediate_timesteps = floor(config.timestep/config.integration_timestep);
next_position = state(1);
next_velocity = state(2);

for k=1:num_of_intermediate_timesteps
    if (next_position <= 0)
        hill_prime = 2*next_position + 1;
    else
        hill_prime = -next_position*(.5*(1+5*next_position^2)^(-1.5)*10*next_position) +...
            (1 + 5*next_position^2)^(-.5);
    end
    acceleration =...
        action/sqrt((1 + hill_prime^2)) - 9.8*hill_prime/(1 + hill_prime^2);
    next_velocity = next_velocity + config.integration_timestep*acceleration;
    next_position = next_position + config.integration_timestep*next_velocity +...
        .5*config.integration_timestep^2*acceleration;
    % Check to see if a penalty state is reached
    if (next_position < -1) || (abs(next_velocity) >= 3)
        fail_flag = true;
        reward = -1;
    % Otherwise, check to see if goal is reached
    elseif (next_position > 1) && (abs(next_velocity) < 3)
        success_flag = true;
        reward = 1;
    else
    % Otherwise, the state is ordinary, carry on
        reward = 0;
    end    
next_state = [next_position, next_velocity];
end

